package peak.can;

import java.util.HashMap;
import java.util.Vector;
import java.util.Random;
import peak.can.basic.TPCANMsg;
import peak.can.basic.TPCANTimestamp;
import peak.can.basic.PCANBasic;
import peak.can.basic.TPCANHandle;
import peak.can.basic.TPCANStatus;

/**
 * The CANSendThread class extends Thread class and is used to send CAN Messages.
 *
 * @version 1.7
 * @LastChange 17/10/2012
 * @author Jonathan Urban/Uwe Wilhelm
 *
 * @Copyright (C) 1999-2012  PEAK-System Technik GmbH, Darmstadt
 * more Info at http://www.peak-system.com
 */
public class CANSendThread extends Thread 
{
    // PCANBasic instance used to call read functions
    private PCANBasic pcanBasic;
    // Collection which stores all connected channels
    private Vector<ChannelItem> connectedChannelCollection = null;
    // Used to send CAN Messages with  Interval
    private int SendInterval;
    
    //Local CAN variables
    private TPCANMsg canMessage = null;
    private TPCANStatus ret;
    // Random Generator
    Random randomGenerator;

    public int getSendInterval()
    {
        return SendInterval;
    }

    public void setSendInterval(int interval)
    {
        SendInterval = interval;
    }

    /**
     *
     * @param can PCANBasic instance used to call read functions
     * @param channels Reference to the collection which stores all connected channels
     */
    public CANSendThread(PCANBasic pcanbasic, Vector<ChannelItem> connectedChannelCollection)
    {
        this.pcanBasic = pcanbasic;
        this.connectedChannelCollection = connectedChannelCollection;
        
         // Create new CAN Message
        this.canMessage = new TPCANMsg();          
        this.canMessage.setID((int)1024); // HEX 400
        this.canMessage.setLength((byte)8); // 8 Byte
        this.canMessage.setType(TPCANMsg.MSGTYPE_STANDARD); // Std. CAN Frame       
        this.randomGenerator = new Random();        
    }  

    protected void finalize() throws Throwable
    {
      // Free local variables
       canMessage = null;
       randomGenerator = null;
     }    
    
    /**
     * Starts thread process
     */
    public void run()
    {
        while (true)
        {
            // Process each connected channel
            for (ChannelItem item : connectedChannelCollection)
            {
                if ((item != MarkAllChannelItem.getInstance()) && (item.getWorking()))
                    // Call the PCANBasic Send Function
                    callAPIFunctionSend(item.getHandle());
            }

            // Sleep Time
            try
            {
                Thread.sleep(SendInterval);
            }
            catch (InterruptedException e)
            {
                return;
            }
        }
    }

    /**
     * Calls the PCANBasic Send Function 
     *
     * @param handle The handle of a PCAN Channel
     */
    public void callAPIFunctionSend(TPCANHandle handle)
    {
        byte Data;
        try
        {
            Data = (byte)randomGenerator.nextInt(249);
            this.canMessage.getData()[0]= Data; // Data...
            this.canMessage.getData()[1]= (byte)(Data + 1);
            this.canMessage.getData()[2]= (byte)(Data + 2);
            this.canMessage.getData()[3]= (byte)(Data + 3);
            this.canMessage.getData()[4]= (byte)(Data + 4);
            this.canMessage.getData()[5]= (byte)(Data + 5);
            this.canMessage.getData()[6]= (byte)(Data + 6);
            this.canMessage.getData()[7]= (byte)(Data + 7);        
            
            // We execute the "Write" function of the PCANBasic
            ret = pcanBasic.Write(handle, canMessage);
            //Process result
            if (ret == TPCANStatus.PCAN_ERROR_OK)
            {
              //Critical Area
              synchronized (Application.token)
              {
                //Put Message In the dataRowCollection
              }
            }

        }
        catch (Exception e)
        {
            System.out.println("CANSendThread Exception:" + e.getMessage());
            e.printStackTrace();
            System.exit(0);
        }
    }
}
